import "dotenv/config";
import SQLite from "better-sqlite3";
import { existsSync } from "fs";
import { Kysely, SqliteDialect } from "kysely";
import { Database, MenuItemUpdate, NewMenuItem, NewOrderState, Order, PaymentMethod } from "../db.ts"; // this is the Database interface we defined earlier

let migration = 0;
const dbName = process.env.DB_PATH || "order.sqlite3";
if (existsSync(dbName)) {
  console.log("Database exists");
} else {
  migration = 1;
}

const sqlite3DB = new SQLite(dbName);

if (migration > 0) {
  sqlite3DB.exec(`
        CREATE TABLE Order_State (
            id TEXT PRIMARY KEY,
            orders TEXT NOT NULL,
            completed INTEGER NOT NULL,
            payment TEXT NOT NULL
        );
        CREATE TABLE Order_Number (
            id TEXT PRIMARY KEY,
            number INTEGER NOT NULL
        );
        CREATE TABLE Menu (
            name TEXT PRIMARY KEY,
            HOT INTEGER,
            COLD INTEGER,
            price INTEGER,
            category TEXT NOT NULL
        );
        INSERT INTO Menu (name, HOT, COLD, category, price) VALUES
            ('아메리카노', 1.5, 2, '커피',NULL),
            ('헤이즐넛 아메리카노', 1.5, 2, '커피',NULL),
            ('바닐라 아메리카노', 1.5, 2, '커피',NULL),
            ('꿀단지 커피', 2.5, 3, '커피',NULL),
            ('곰다방커피', NULL, 2, '커피',NULL),
            ('콜드브루', NULL, 2, '커피',NULL),
            ('카페모카', 2, 2.5, '커피',NULL),
            ('카페라떼', 3, 3.5, '카페라떼',NULL),
            ('헤이즐넛 카페라떼', 3, 3.5, '카페라떼',NULL),
            ('바닐라 카페라떼', 3, 3.5, '카페라떼',NULL),
            ('초코라떼', 3, 3.5, '카페라떼',NULL),
            ('딸기라떼', 3, 3.5, '카페라떼',NULL),
            ('사과유자차', 3, 3.5, '차',NULL),
            ('꿀유자차', 2, 2.5, '차',NULL),
            ('폴라복숭아', NULL, 2.5, '차',NULL),
            ('허니자몽 블랙티', 2.5, 3, '차',NULL),
            ('오미자 차', 2, 2.5, '차',NULL),
            ('달곰상곰 딸기레몬', 3.5, 3.5, '탄산',NULL),
            ('청포도 에이드', 3, 3, '탄산',NULL),
            ('자몽에이드', 3, 3, '탄산',NULL),
            ('레몬에이드', 3, 3, '탄산',NULL),
            ('오미자 에이드', 3, 3, '탄산',NULL),
            ('허니브레드', NULL, NULL, '빵', 3),
            ('소금빵', NULL, NULL, '빵', 3),
            ('크로크무슈', NULL, NULL, '빵', 3.5);
    `);
}

const dialect = new SqliteDialect({
  database: sqlite3DB,
});

const db = new Kysely<Database>({
  dialect,
});

export async function requestOrderNumber(): Promise<number> {
  const date = new Date().toISOString().split("T")[0];

  return await db.transaction().execute(async trx => {
    const order_number = await trx.selectFrom("Order_Number")
      .where("id", "==", date)
      .selectAll()
      .executeTakeFirst();
    if (!order_number) {
      await trx.insertInto("Order_Number")
        .values({
          id: date,
          number: 1,
        })
        .execute();
      return 1;
    }
    const num = order_number.number + 1;
    await trx.updateTable("Order_Number")
      .set("number", num)
      .where("id", "==", date)
      .execute();
    return num;
  });
}

export async function saveOrder(order: Order[], payment: PaymentMethod) {
  const date = new Date().toISOString().split("T")[0];
  const num = await requestOrderNumber();
  const id = `${date}/${num}`;
  await db.insertInto("Order_State")
    .values({
      id,
      orders: JSON.stringify(order),
      completed: 0,
      payment,
    } as NewOrderState)
    .execute();
  return id;
}

export async function loadOrder() {
  const orders = (await db.selectFrom("Order_State")
    .selectAll()
    .orderBy(["completed", "id desc"])
    .execute())
    .map(order => ({
      ...order,
      orders: JSON.parse(order.orders as unknown as string) as Order[],
    }));

  return orders;
}

export async function clearOrder() {
  await db.deleteFrom("Order_State")
    .execute();
}

export async function cancelOrder(order_id: string) {
  await db.deleteFrom("Order_State")
    .where("id", "==", order_id)
    .execute();
}

export async function completeOrder(uid: string, completed: 0 | 1 = 1) {
  await db.updateTable("Order_State")
    .set("completed", completed)
    .where("id", "==", uid)
    .execute();
}

export async function loadOrderById(uid: string) {
  const order = await db.selectFrom("Order_State")
    .where("id", "==", uid)
    .selectAll()
    .executeTakeFirst();
  return order
    ? {
      ...order,
      orders: JSON.parse(order.orders as unknown as string) as Order[],
    }
    : null;
}

export async function addNewMenuItem(item: NewMenuItem) {
  await db.insertInto("Menu")
    .values(item)
    .execute();
}

export async function listMenuItems() {
  return await db.selectFrom("Menu")
    .selectAll()
    .orderBy(["category", "name"])
    .execute();
}

export async function deleteMenuItem(name: string) {
  await db.deleteFrom("Menu")
    .where("name", "==", name)
    .execute();
}

export async function updateMenuItem(name: string, item: MenuItemUpdate) {
  await db.updateTable("Menu")
    .set(item)
    .where("name", "==", name)
    .execute();
}
