import { Router } from "express";
import { addNewOrderSchema } from "src/schema.ts";
import * as api from "./db.ts";
import { errorWrapper } from "./error_handler.ts";

const router = Router();

const listeners = new Set<() => void>();

function addOrderListener(listener: () => void) {
  listeners.add(listener);
}

function removeOrderListener(listener: () => void) {
  listeners.delete(listener);
}

function emitOrder() {
  for (const listener of listeners) {
    listener();
  }
}

router.get(
  "/order",
  errorWrapper(async (req, res) => {
    res.json(await api.loadOrder());
  }),
);

router.get(
  "/order/notify",
  errorWrapper(async (req, res) => {
    res.setHeader("Content-Type", "text/event-stream");
    res.setHeader("Cache-Control", "no-cache");
    res.setHeader("Connection", "keep-alive");
    res.flushHeaders();

    const listener = () => {
      res.write(
        `data: ${
          JSON.stringify({
            change: "order",
          })
        }\n\n`,
        (err) => {
          if (err) {
            // we could not send the event to the client anymore
            // so we could not call next()
            // only log the error
            console.error(err);
          }
        },
      );
    };

    addOrderListener(listener);

    req.on("close", () => {
      removeOrderListener(listener);
    });
  }),
);

router.get(
  "/order/:id",
  errorWrapper(async (req, res) => {
    const order = await api.loadOrderById(req.params.id);
    if (order) {
      res.json(order);
    } else {
      res.status(404).json({ error: "Order not found" });
    }
  }),
);

router.post(
  "/order",
  errorWrapper(async (req, res) => {
    const orderReq = addNewOrderSchema.parse(req.body);
    const orders = orderReq.orders;
    const payment = orderReq.payment;
    const id = await api.saveOrder(orders, payment);
    emitOrder();
    res.json({ id });
  }),
);

router.delete(
  "/order/:id",
  errorWrapper(async (req, res) => {
    await api.cancelOrder(req.params.id);
    emitOrder();
    res.json({ success: true });
  }),
);

router.post(
  "/order/:id/complete",
  errorWrapper(async (req, res) => {
    await api.completeOrder(req.params.id, 1);
    emitOrder();
    res.json({ success: true });
  }),
);

router.post(
  "/order/:id/cancel-complete",
  errorWrapper(async (req, res) => {
    await api.completeOrder(req.params.id, 0);
    emitOrder();
    res.json({ success: true });
  }),
);

router.delete(
  "/order",
  errorWrapper(async (req, res) => {
    await api.clearOrder();
    emitOrder();
    res.json({ success: true });
  }),
);

export default router;
